'use strict';

angular.module('fadel',[])
    .config(function($locationProvider) {
        $locationProvider.html5Mode(true);
    })
    .run(['$document', function($document) {
        $document[0].body.style.visibility = 'visible';
    }])
    .constant('formatTypeCode', 20)
    .constant('territoryTypeCode', 30)
    .controller('AppController', ['$timeout', '$q', '$scope', '$location', '$window', 'getAssetClearanceSearchCriteria', 'requestRightAttributes', 'requestAssetClearance', 'requestAssetThumbnail', 'requestAssetRightsAvailabilityAnswers', 'formatTypeCode', 'territoryTypeCode', 'global', function($timeout, $q, $scope, $location, $window, getAssetClearanceSearchCriteria, requestRightAttributes, requestAssetClearance, requestAssetThumbnail, requestAssetRightsAvailabilityAnswers, formatTypeCode, territoryTypeCode, global) {
        var locationSearch = $location.search();
        var redirectBoxUrl = locationSearch['redirect_to_box_url'];
        $scope.fileId = locationSearch['file_id'];
        $scope.fileName = locationSearch['file_name'];

        function getRightAttributes(callback){
            $q.all([requestRightAttributes(formatTypeCode), requestRightAttributes(territoryTypeCode)]).then(function(attributes){
                $scope.formatRightAttributes = attributes[0];
                $scope.territoryRightAttributes = attributes[1];
                $scope.attributes = {};
                $scope.attributes.format = $scope.formatRightAttributes.length == 0 ? null : $scope.formatRightAttributes[0];
                $scope.attributes.territory = $scope.territoryRightAttributes.length == 0 ? null : $scope.territoryRightAttributes[0];
                callback(true);
            }).catch(function(response){
                if(response.status == 403){
                    callback(false);
                } else {
                    showError(response);
                }
            });
        }

        function showError(response){
            $window.alert('ERROR Server request failed with status code: ' + response.status + ', message: ' + (response.data || ''));
        }

        $scope.setError = function(message){
            $scope.error = message;
        }

        $scope.login = function(username, password){
            //use '_' as a workaround: if we pass '' then the server api can hang
            global.authCredentials = {
                username : username || '_',
                password : password || '_'
            };
            getRightAttributes(function(logged){
                if(logged){
                    $window.localStorage.setItem("authCredentials", JSON.stringify(global.authCredentials));
                    $scope.viewType = "clearance";
                } else {
                    $scope.setError("Authentication failed");
                }
            });
        }

        $scope.close = function(){
            $window.location.href = redirectBoxUrl;
        }

        $scope.getDates = {};
        $scope.getClearance = function(){
            $scope.closeExceptionsDetails();
            $scope.clearance = null;
            var fromDate = $scope.getDates.from();
            var toDate = $scope.getDates.to();
            if(!$scope.attributes.format || !$scope.attributes.territory || !fromDate || !toDate){
                $timeout(function(){
                    $scope.setError('Please, fill all fields');
                }, 0);
            } else {
                var searchCriteria = getAssetClearanceSearchCriteria($scope.fileId, $scope.fileName, $scope.attributes.format.rightRelationshipId, $scope.attributes.territory.rightRelationshipId, fromDate.toDate(), toDate.toDate());
                requestAssetClearance(searchCriteria).then(function(clearance){
                    $scope.searchCriteria = searchCriteria;
                    $scope.clearance = {
                        typeName : clearance.typeName,
                        availableAnswerListUri : clearance.availableAnswerListUri,
                        notAvailableAnswerListUri : clearance.notAvailableAnswerListUri
                    };
                    if(clearance.thumbNailUri){
                        requestAssetThumbnail(clearance.thumbNailUri).then(function(thumbnail){
                            $scope.clearance.thumbnail = thumbnail;
                        })
                    }
                }).catch(function(response){
                    $scope.clearance = {typeName : 'COULD NOT GET DATA'};
                });
            }
        }

        $scope.closeExceptionsDetails = function(){
            $scope.exceptionsDetails = null;
        }

        $scope.getExceptionsDetails = function(){
            $q.all([requestAssetRightsAvailabilityAnswers($scope.clearance.availableAnswerListUri, $scope.searchCriteria, true), requestAssetRightsAvailabilityAnswers($scope.clearance.notAvailableAnswerListUri, $scope.searchCriteria, false)]).then(function(results){
                var answers = results[0].concat(results[1]);
                var formats = [];
                var territories = [];
                var availability = [];
                for(var i = 0; i < answers.length; i++){
                    var answer = answers[i];
                    var territoryIndex = territories.indexOf(answer.territory);
                    if(territoryIndex == -1){
                        territoryIndex = territories.length;
                        territories.push(answer.territory);
                        availability.push([]);
                    }
                    var formatIndex = formats.indexOf(answer.format);
                    if(formatIndex == -1){
                        formatIndex = formats.length;
                        formats.push(answer.format);
                    }
                    availability[territoryIndex][formatIndex] = answer.available;
                }
                $scope.exceptionsDetails = {
                    formats : formats,
                    territories : territories,
                    availability : availability
                }
            }).catch(function(response){
                showError(response);
            });
        }

        //------------------init app-----------------------
        var credentials = JSON.parse($window.localStorage.getItem("authCredentials"));
        if(credentials == null){
            $scope.viewType = "login";
            $scope.appLoaded = true;
        } else {
            global.authCredentials = credentials;
            getRightAttributes(function(logged){
                $scope.viewType = logged ? "clearance" : "login";
                $scope.appLoaded = true;
            });
        }
    }])
    .directive('hover', ["$window", function($window) {
        return {
            restrict: 'E',
            transclude : true,
            link: function(scope, element, attrs) {
                if(!scope.enabled){
                    return;
                }
                var item = element.find('div');
                $window.hoverClick = function(){
                    scope.$apply(function(){
                        scope.click();
                        item.popover('hide');
                    })
                }
                item.popover({content : '<div>To view details, <a href onclick="hoverClick()">click here</a></div>', container: item, trigger: "hover", html: true, placement : "top"});
            },
            scope: {click : '=', enabled : '='},
            template: '<ng-transclude></ng-transclude>'
        };
    }])
    .directive('dateInput', function() {
        return {
            restrict: 'E',
            link: function(scope, element, attrs) {
                var dateTimePicker = element;
                dateTimePicker.datetimepicker({format : 'MM/DD/YYYY'});
                scope.getDate = function (){
                    return dateTimePicker.data("DateTimePicker").date();
                };
            },
            replace : true,
            scope: {getDate : '=', placeholder : '@'},
            template: '<div class="input-group date"><input type="text" class="form-control" placeholder="{{placeholder}}"/><span class="input-group-addon"><span class="glyphicon glyphicon-calendar"></span></span></div>'
        };
    })
    .factory('global', ['$window', function() {
        return {};
    }])
    .factory('getAssetClearanceSearchCriteria', ['formatTypeCode', 'territoryTypeCode', function(formatTypeCode, territoryTypeCode) {
        return function(assetId, assetName, formatAttribute, territoryAttribute, dateFrom, dateTo){
            var selected = {};
            selected[formatTypeCode] = {};
            selected[formatTypeCode][formatAttribute] = {};
            selected[territoryTypeCode] = {};
            selected[territoryTypeCode][territoryAttribute] = {};
            var partial = {};
            partial[formatTypeCode] = {};
            partial[territoryTypeCode] = {};
            var all = {};
            all[formatTypeCode] = "false";
            all[territoryTypeCode] = "false";

            var selectedAssets = {};
            selectedAssets[assetId] = {
                "externalIdentifier": assetId,
                "name": assetName
            };

            var data = {
                "assets": {},
                "rights": {
                    "selected": selected,
                    "partial": partial,
                    "all": all
                },
                "inDateFrom": dateFrom.getTime(),
                "outDateFrom": dateTo.getTime(),
                "externalAssets": {
                    "selected": selectedAssets,
                    "all": "false"
                },
                "rightTypeLst": [
                    {
                        "lookupType": "FPI_RIGHT_DEFINITION_TYPES",
                        "fieldOrder": 10,
                        "lookupCode": '' + formatTypeCode,
                        "displayName": "Format"
                    },
                    {
                        "lookupType": "FPI_RIGHT_DEFINITION_TYPES",
                        "fieldOrder": 20,
                        "lookupCode": '' + territoryTypeCode,
                        "displayName": "Territory"
                    }
                ]
            };
            return data;
        }
    }])
    .factory('requestAssetRightsAvailabilityAnswers', ['api', '$q', 'formatTypeCode', 'territoryTypeCode', function(api, $q, formatTypeCode, territoryTypeCode) {
        return function(uri, searchCriteria, available){
            if(!uri){
                return $q.when([]);
            }
            return api.do(uri, searchCriteria, 'POST', 'application/json').then(function(response){
                var results = [];
                var answers = response.data.assetAnswer;
                for(var i = 0; i < answers.length; i++){
                    var answer = answers[i].answer;
                    results.push({
                        format : answer[formatTypeCode].right.description,
                        territory : answer[territoryTypeCode].right.description,
                        available : available
                    });
                }
                return results;
            });
        }
    }])
    .factory('requestAssetClearance', ['api', function(api) {
        return function(searchCriteria){
            return api.do(api.baseUri + 'clearanceWS/assetClearance', searchCriteria, 'POST', 'application/json').then(function(response){
                return response.data.clearance[0];
            });
        }
    }])
    .factory('requestRightAttributes', ['api', function(api) {
        return function(typeCode){
            var results = [];
            function addAttributes(parentNames, attributes){
                for(var i = 0; i < attributes.length; i++){
                    var attribute = attributes[i];
                    var names = parentNames.slice();
                    names.push(attribute.right.description);
                    results.push({
                        rightRelationshipId : attribute.rightRelationshipId,
                        name : names.join(' > ')
                    });
                    addAttributes(names, attribute.childrenLst);
                }
            }
            return api.do(api.baseUri + 'rightWS/rightAttributes/' + typeCode, {}, 'POST', 'application/x-www-form-urlencoded').then(function(response){
                addAttributes([], response.data.attribute);
                return results;
            })
        }
    }])
    .factory('requestAssetThumbnail', ['api', function(api) {
        return function(uri){
            return api.do(uri, null, 'GET', 'application/x-www-form-urlencoded').then(function(response){
                return response.data.thumbnNail;
            })
        }
    }])
    .factory('api', ['$http', 'global', function($http, global) {
        return {
            baseUri : 'http://ec2-54-83-24-246.compute-1.amazonaws.com:8082/ipmSuite5/ws/V1/',
            do : function(uri, data, method, contentType){
                return $http({
                    method : 'POST',
                    url: 'api',
                    data : {
                        uri : uri,
                        data : data,
                        method : method,
                        contentType : contentType
                    },
                    headers : {
                        username : global.authCredentials.username,
                        password : global.authCredentials.password
                    }
                });
            }
        }
    }])
;
